//class PSWThread
function PSWThread(psw)
{
	this.psw = psw;
	this.observerService = Components.classes["@mozilla.org/observer-service;1"]
	  .getService(Components.interfaces.nsIObserverService);
 	this.addObserver();
 	this.pswPref = new PSWPref();
	this.columnHandler = null;
	this.makeColumnHandler();
	this.pswPref.addPrefListener(this, this.regardNoneAsNormal,
								 this.pswPref.PrefRegardNoneAsNormal);
	this.pswPref.addPrefListener(this, this.setCycleCell,
								 this.pswPref.PrefCycleCell);
	this.switchIconTheme();
	this.regardNoneAsNormal();
	this.setCycleCell();
	try {
		this.setupColumn(); // for TB3
	} catch(e) {
		//TB2
	}
	
	GetThreadTree().addEventListener("mousedown", this.OnMouseClick, true);
}

PSWThread.prototype.OnMouseClick = function(event)
{
	var pswCol = document.getElementById("colPSWPriorityIcon");
	pswCol.setAttribute("swDir", "descending"); //reset cycle direction

	if (event.button != 0) return;

	if (!event.altKey) return;
	var t = event.originalTarget;
	if (t.localName != "treechildren") return;

	var row = new Object;
	var col = new Object;
	var childElt = new Object;

	var tree = GetThreadTree();
	// figure out what cell the click was in
	tree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, childElt);
	if (row.value == -1) return;

	if (col.value.id == "colPSWPriorityIcon")
	  pswCol.setAttribute("swDir", "ascending"); //cycle in ascending direction
}

PSWThread.prototype.regardNoneAsNormal = function()
{
	this.columnHandler.regardAsNormal = this.pswPref.regardNoneAsNormal();
}

PSWThread.prototype.setCycleCell = function()
{
	this.columnHandler.enableCycleCell = this.pswPref.cycleCell();
}

PSWThread.prototype.switchIconTheme = function()
{
	var look = this.pswPref.getIconTheme();
	if (look) {
		document.getElementById("colPSWPriorityIcon").setAttribute("look", look);
	} else {
		document.getElementById("colPSWPriorityIcon").removeAttribute("look");
	}
}

PSWThread.prototype.setupColumn = function(aMsgFolder, aTopic, aData)
{
	gDBView.addColumnHandler("colPSWPriorityIcon", this.columnHandler);
	this.sortColumn();
	this.psw.setDisplayedPriorityToToolbar();
}

PSWThread.prototype.addObserver = function()
{
	var observer = new PSWObserver(this, this.setupColumn);
	this.observerService.addObserver(observer, "MsgCreateDBView", false);
}

PSWThread.prototype.makeColumnHandler = function()
{
	this.columnHandler = {
	  regardAsNormal: false, //custom
	  enableCycleCell: true, //custom
	  owner: this, //custom
	  look: this.pswPref.getIconTheme(),
	  getCellText: function(row, col) {
		  return "";
	  },
	  getCellValue: function(row, col) {
		  var key = gDBView.getKeyAt(row);
		  var folder = gDBView.getFolderForViewIndex(row);
		  var hdr = folder.GetMessageHeader(key);
//		  var hdr = gDBView.db.GetMsgHdrForKey(key);
		  var priority = hdr.getUint32Property("priority");
		  return priority.toString();
	  },
	  getSortStringForRow: function(hdr) {return "";},
	  isString: function() {return false;},
	  getRowProperties: function(row,props){
		  var prop = this.getPSWRowProperty(row);
		  var atmserv=Components.classes["@mozilla.org/atom-service;1"].
			getService(Components.interfaces.nsIAtomService);
		  props.AppendElement(atmserv.getAtom(prop));
	  },
	  getCellProperties: function(row, col, props){
		  var prop = this.getPSWRowProperty(row);
		  var atmserv=Components.classes["@mozilla.org/atom-service;1"].
			getService(Components.interfaces.nsIAtomService);
		  props.AppendElement(atmserv.getAtom(prop));
	  },
	  cycleCell: function(row, col){
	  	if (!this.enableCycleCell) return;
		  var key = gDBView.getKeyAt(row);
		  var folder = gDBView.getFolderForViewIndex(row);
		  var hdr = folder.GetMessageHeader(key);
		  //var hdr = gDBView.db.GetMsgHdrForKey(key);
		  var priority = hdr.getUint32Property("priority");
		  if (priority < PSWMsgPriorityValue.none)
			priority = PSWMsgPriorityValue.none;
		  
		  var elem = document.getElementById("colPSWPriorityIcon");
		  if (elem.getAttribute("swDir") == "ascending")
			priority = PSWMsgPriorityValue.upCycle(priority);
		  else
			priority = PSWMsgPriorityValue.downCycle(priority);
		  this.owner.notifyRowUpdate(hdr, priority, row);
		  if (this.owner.pswPref.useAutoCommit()) {
			  if (hdr.folder.server.type == "imap" || !this.owner.pswPref.commitOnlyForImap()) {
				  var committer = new PSWCommitter();
				  committer.append(hdr);
				  committer.commit();
			  }
		  }
	  },
	  getImageSrc: function(row, col) {return null;},
	  getSortLongForRow: function(hdr) {
		  var priority = hdr.getUint32Property("priority");
		  if (this.regardAsNormal && priority <= PSWMsgPriorityValue.none)
			priority = PSWMsgPriorityValue.normal;
		  return priority * -1;
	  },
	  getPSWRowProperty: function(row) {
		  var key = gDBView.getKeyAt(row);
		  var folder = gDBView.getFolderForViewIndex(row);
		  var hdr = folder.GetMessageHeader(key);
//		  var hdr = gDBView.db.GetMsgHdrForKey(key);
		  var priority = hdr.getUint32Property("priority");
		  var prop = 'no';
		  switch (priority) {
			case PSWMsgPriorityValue.notSet: case PSWMsgPriorityValue.none:
			  prop='no';
			  break;
			case PSWMsgPriorityValue.lowest:
			  prop='lowest';
			  break;
			case PSWMsgPriorityValue.low:
			  prop='low';
			  break;
			case PSWMsgPriorityValue.normal:
			  prop='nor';
			  break;
			case PSWMsgPriorityValue.high:
			  prop='high';
			  break;
			case PSWMsgPriorityValue.highest:
			  prop='highest';
			  break;
			default:
			  prop='no';
		  }
		  if (this.look) prop = prop + "-" + this.look;
		  return prop;
	  }
	};
}

PSWThread.prototype.notifyRowUpdate = function(hdr, priority, row)
{
	this.psw.changePriorityForHdr(hdr, priority, row);
	this.psw.setDisplayedPriorityToToolbar();
}

PSWThread.prototype.sortColumn = function()
{
	var sortDir = document.getElementById("colPSWPriorityIcon").getAttribute("sortDirection");

	if (sortDir == "ascending") {
		//MsgSortThreadPane('byPriority');
		MsgSortThreadPane('byCustom');
		MsgSortAscending();
	}	else if (sortDir == "descending") {
		//MsgSortThreadPane('byPriority');
		MsgSortThreadPane('byCustom');
		MsgSortDescending();
	}
}

//class PSWTreeUpdater
function PSWThreadUpdater(parentWindow)
{
	if (parentWindow)
	  this.tree = parentWindow.GetThreadTree();
	else
	  this.tree = GetThreadTree();
	this.treebox = this.tree.boxObject.QueryInterface(Components.interfaces.nsITreeBoxObject);
}

PSWThreadUpdater.prototype.updateCell = function(row, col)
{
	this.treebox.invalidateCell(row, col);
}

PSWThreadUpdater.prototype.updateColumn = function(col)
{
	this.treebox.invalidateColumn(col);
}

PSWThreadUpdater.prototype.updateRow = function(row)
{
	this.treebox.invalidateRow(row);
}

PSWThreadUpdater.prototype.update = function()
{
	this.treebox.invalidate();
}
